#!/usr/bin/env python3

import re, sys
from math import sqrt

def read_file( fn ):
    fd = open( fn, 'r' )
    ti = fd.read().rstrip()
    fd.close
    ti = re.sub( r'^\s*#.*\n', '', ti, re.M )
    return ti.split("\n")

## Main

maxdiff = 0
fpair = sys.argv[1:3]
tpair = [ read_file( fn ) for fn in fpair ]
if len(tpair[0])!=len(tpair[1]):
    raise Exception( "tables differ in size" )
for i in range(len(tpair[0])):
    dpair = [ re.split(r'\s+', t[i]) for t in tpair]
    if len(dpair[0])!=len(dpair[1]):
        raise Exception( "lines %i differ in size" % i )
    for j in range(5):
        if dpair[0][j]!=dpair[1][j]:
            raise Exception( "lines %i differ in parameter %i" % (i,j) )
    xypair = [ [ float(d[8]), float(d[9]) ] for d in dpair ]
    out_of_range = False
    for x,y in xypair:
        if abs(x)>1e166 or abs(y)>1e166:
            out_of_range = True
            break
    if out_of_range:
        print( tpair[0][i] )
        print( tpair[1][i] )
        print( "result too large" )
        continue
    apair = [ sqrt(x**2+y**2) for x,y in xypair ]
    aval = (apair[0] + apair[1])/2
    diff = max( abs(xypair[0][0]-xypair[1][0]), abs(xypair[0][1]-xypair[1][1]) ) / aval
    maxdiff = max( maxdiff, diff )
    if diff>3.e-12:
        print( " ".join(dpair[0]+dpair[1][8:]+["%g" % diff]) )
print( "# maxdiff = %g" % maxdiff )



#!/usr/bin/env python3

# setup_icosahedron.py:
# Compute vertex coordinates of a regular icosahedron.
# Auxiliary program, used during implementation of icosahedron form factor in BornAgain.
# Joachim Wuttke, Forschungszentrum Jülich, 2016

from math import sqrt,sin,cos,pi

def distance(s,t):
    return sqrt((s[0]-t[0])**2+(s[1]-t[1])**2+(s[2]-t[2])**2)

c=sqrt(3)/2
q=sqrt(5)
ss=c
cc=.5

# a=pow(3*(3-q)/5, 1/3.) # edge for V=1
a=1.

R2=(5+q)/8*a**2
rb=a/sqrt(3)
b=a*sqrt((7+3*q)/24) # sqrt(R2-rb**2)
rd2=a**2*(3+q)/6 # (4*R2-a**2)/2/(1+cc)
rd=sqrt(rd2)
d=a*sqrt((3-q)/24) #sqrt(R2-rd2)

h=rb+sqrt(rb**2-(0.5*a)**2)
vol = 20 * h*a*b/6
print("a=%g, R=%g, rb=%g, rd=%g, b=%g, d=%g" % (a, sqrt(R2), rb, rd, b, d))
print("check P14=%g" % (sqrt( (rd*cc-rb)**2 + (rd*ss)**2 + (d-b)**2 )))
print("h=%g volume=%20.16g" % (h, vol) )

p=[None for i in range(12)]
p[ 0] = [ rb,          0,            -b]
p[ 1] = [-rb*cos(pi/3),+rb*sin(pi/3),-b]
p[ 2] = [-rb*cos(pi/3),-rb*sin(pi/3),-b]
p[ 3] = [-rd,          0,            -d]
p[ 4] = [ rd*cos(pi/3),+rd*sin(pi/3),-d]
p[ 5] = [ rd*cos(pi/3),-rd*sin(pi/3),-d]
p[ 6] = [ rd,          0,            +d]
p[ 7] = [-rd*cos(pi/3),+rd*sin(pi/3),+d]
p[ 8] = [-rd*cos(pi/3),-rd*sin(pi/3),+d]
p[ 9] = [-rb,          0,            +b]
p[10] = [ rb*cos(pi/3),+rb*sin(pi/3),+b]
p[11] = [ rb*cos(pi/3),-rb*sin(pi/3),+b]

for i in range(len(p)):
    print("        {%20.16g*a,%20.16g*a,%20.16g*a}," % (p[i][0], p[i][1], p[i][2]))

for i in range(len(p)):
    print("p%2i(%20.16g,%20.16g,%20.16g) at R=%20.16g" %
          (i, p[i][0], p[i][1], p[i][2], distance(p[i],[0,0,0])))

distcoll = {}
for i in range(len(p)):
    for j in range(i+1,len(p)):
        dist = distance(p[i],p[j])
        print("p%2i-%2i = %20.17g" % (i, j, dist))
        dist=round(dist*1e15)/10**15
        if not dist in distcoll:
            distcoll[dist] = 1
        else:
            distcoll[dist] += 1

for dist in sorted(distcoll.keys()):
    print( "%2i times %20.16g" % (distcoll[dist],dist) )



# script to plot the form factor
# after the ROOT window with plots appear,
# just save the picture in that format which you need

import ROOT
from libBornAgainCore import *

# define a form factor, I recommend a 10--20 nm diameter
# vary H
ff05 =  FormFactorPyramid(2.5*nanometer, 5.0*nanometer, 60*degree)
ff10 =  FormFactorPyramid(5.0*nanometer, 5.0*nanometer, 60*degree)
ff15 =  FormFactorPyramid(7.5*nanometer, 5.0*nanometer, 60*degree)
# vary alpha
#ff05 =  FormFactorPyramid(5.0*nanometer, 5.0*nanometer, 50*degree)
#ff10 =  FormFactorPyramid(5.0*nanometer, 5.0*nanometer, 65*degree)
#ff15 =  FormFactorPyramid(5.0*nanometer, 5.0*nanometer, 80*degree)

# zero q vector
zero = cvector_t(0,0,0)
# volume of particles
V05 = abs(ff05.evaluate_for_q(zero))
V10 = abs(ff10.evaluate_for_q(zero))
V15 = abs(ff15.evaluate_for_q(zero))


# number of bins for qx, qy, qz
# I recommend to put at least 400 for a better picture quality
nqy = 400
nqz = 400
nqx = 400

# minimum and maximum values for qx, qy qz
qymin = -2.0
qymax = 2.0
qzmin = -2.0
qzmax = 2.0
qxmin = -2.0
qxmax = 2.0

# first and last bin for the projection slice
# if number of bins=400, then fbin should be set to 200 and lbin to 201
#fbin=50
#lbin=51

# step for qx, qy, qz
stepqy = (qymax - qymin)/(nqy-1)
stepqz = (qzmax - qzmin)/(nqz-1)
stepqx = (qxmax - qxmin)/(nqx-1)

# define style of the diagram
ROOT.gROOT.SetStyle("Plain")
ROOT.gStyle.SetOptStat(0);
ROOT.gStyle.SetOptTitle(1);
ROOT.gStyle.SetLabelSize(0.06, "xyz");
ROOT.gStyle.SetTitleSize(0.06, "xyz");
ROOT.gStyle.SetTitleFontSize(0.1);
#ROOT.gStyle.SetPadRightMargin(0.19)
ROOT.gStyle.SetPadLeftMargin(0.18)
ROOT.gStyle.SetPadBottomMargin(0.18)
ROOT.gStyle.SetTitleX(0.3);
#ROOT.gStyle.SetTitleW(0.5);


#t = ROOT.TText()
# create ROOT histograms 
h05 = ROOT.TH2D("h05nm","H = 2.5 nm",nqy,qymin,qymax, nqz, qzmin, qzmax)
h10 = ROOT.TH2D("h10nm","H = 5.0 nm",nqy,qymin,qymax, nqz, qzmin, qzmax)
h15 = ROOT.TH2D("h15nm","H = 7.5 nm",nqy,qymin,qymax, nqz, qzmin, qzmax)
#h05 = ROOT.TH2D("a50"," #alpha = 50^{o} ",nqy,qymin,qymax, nqz, qzmin, qzmax)
#h10 = ROOT.TH2D("a65"," #alpha = 65^{o} ",nqy,qymin,qymax, nqz, qzmin, qzmax)
#h15 = ROOT.TH2D("a80"," #alpha = 80^{o} ",nqy,qymin,qymax, nqz, qzmin, qzmax)

# and fill them with the values
for i in range(nqy):
	qy = qymin + i*stepqy
	for j in range(nqz):
		qz = qzmin + j*stepqz
		k = cvector_t(0,qy,qz)
		h05.Fill(qy,qz,(abs(ff05.evaluate_for_q(k))/V05)**2)
		h10.Fill(qy,qz,(abs(ff10.evaluate_for_q(k))/V10)**2)
		h15.Fill(qy,qz,(abs(ff15.evaluate_for_q(k))/V15)**2)



# create a ROOT canvas and put all plots on it
c = ROOT.TCanvas("FormfactorSphere","Formfactor Sphere", 1200,400)

c.Divide(3,1)
h05.SetMinimum(1e-7)
#h05.SetMaximum(V05**2)
h05.SetMaximum(1)
h10.SetMinimum(1e-7)
#h10.SetMaximum(V10**2)
h10.SetMaximum(1)
h15.SetMinimum(1e-7)
#h15.SetMaximum(V15**2)
h15.SetMaximum(1)
h05.SetContour(50)
h10.SetContour(50)
h15.SetContour(50)
#h05.GetZaxis().SetTitle(" |F|^{2} ")
#h10.GetZaxis().SetTitle(" |F|^{2} ")
#h15.GetZaxis().SetTitle(" |F|^{2} ")


c.cd(1)
ROOT.gPad.SetLogz()
h05.GetXaxis().SetTitle(" q_{y} [nm^{-1}] ")
h05.GetXaxis().CenterTitle()
h05.GetXaxis().SetTitleOffset(1.1)
h05.GetYaxis().SetTitle(" q_{z} [nm^{-1}] ")
h05.GetYaxis().CenterTitle()
h05.GetYaxis().SetTitleOffset(1.4)
h05.GetZaxis().SetTitleOffset(1.3)
h05.Draw("col")
#t.SetNDC(1)
#t.SetTextSize(8.0e-2)
#t.DrawText(0.1, 0.01, "H = 5 nm")

c.cd(2)
ROOT.gPad.SetLogz()
h10.GetXaxis().SetTitle(" q_{y} [nm^{-1}] ")
h10.GetXaxis().CenterTitle()
h10.GetXaxis().SetTitleOffset(1.1)
h10.GetYaxis().SetTitle(" q_{z} [nm^{-1}] ")
h10.GetYaxis().CenterTitle()
h10.GetYaxis().SetTitleOffset(1.4)
h10.GetZaxis().SetTitleOffset(1.3)
h10.Draw("col")
#t.SetNDC(1)
#t.SetTextSize(8.0e-2)
#t.DrawText(0.1, 0.01, "H = 10 nm")

c.cd(3)
ROOT.gPad.SetLogz()
h15.GetXaxis().SetTitle(" q_{y} [nm^{-1}] ")
h15.GetXaxis().CenterTitle()
h15.GetXaxis().SetTitleOffset(1.1)
h15.GetYaxis().SetTitle(" q_{z} [nm^{-1}] ")
h15.GetYaxis().CenterTitle()
h15.GetYaxis().SetTitleOffset(1.4)
h15.GetZaxis().SetTitleOffset(1.3)
h15.Draw("col")
#t.SetNDC(1)
#t.SetTextSize(8.0e-2)
#t.DrawText(0.1, 0.01, "H = 15 nm")

# draw projections
#c.cd(4)
#ROOT.gPad.SetLogy()
#p05 = h05.ProjectionX("p05", fbin, lbin, 'o')
#p05.GetYaxis().SetTitle(" |F|^{2} ")
#p05.GetYaxis().SetTitleOffset(1.3)
#p05.Draw()
#t.DrawText(0.1, 0.01, "d")

#c.cd(5)
#ROOT.gPad.SetLogy()
#p10 = h10.ProjectionX("p10", fbin, lbin, 'o')
#p10.GetYaxis().SetTitle(" |F|^{2} ")
#p10.GetYaxis().SetTitleOffset(1.3)
#p10.Draw()
#t.DrawText(0.1, 0.01, "e")

#c.cd(6)
#ROOT.gPad.SetLogy()
#p15 = h15.ProjectionX("p15", fbin, lbin, 'o')
#p15.GetYaxis().SetTitle(" |F|^{2} ")
#p15.GetYaxis().SetTitleOffset(1.3)
#p15.Draw()
#t.DrawText(0.1, 0.01, "f")



c.Update()
ROOT.gApplication.Run()






# script to plot the form factor
# after the ROOT window with plots appear,
# just save the picture in that format which you need

import ROOT
from libBornAgainCore import *

# define a form factor, I recommend a 10--20 nm diameter
#ff = FormFactorFullSphere(10.0*nanometer)
#ff =  FormFactorSphere(10.0*nanometer, 13.0*nanometer)
ff =  FormFactorPyramid(13*nanometer, 10.0*nanometer, 60*degree)


# volume
# I suggest, that the form factor evaluated for q=0 gives the volume
# please correct if not
zero = cvector_t(0,0,0)
V = abs(ff.evaluate_for_q(zero))


# number of bins for qx, qy, qz
# I recommend to put at least 400 for a better picture quality
nqy = 400
nqz = 400
nqx = 400

# minimum and maximum values for qx, qy qz
qymin = -2.0
qymax = 2.0
qzmin = -2.0
qzmax = 2.0
qxmin = -2.0
qxmax = 2.0

# first and last bin for the projection slice
# if number of bins=400, then fbin should be set to 200 and lbin to 201
#fbin=200
#lbin=201

# step for qx, qy, qz
stepqy = (qymax - qymin)/(nqy-1)
stepqz = (qzmax - qzmin)/(nqz-1)
stepqx = (qxmax - qxmin)/(nqx-1)

# define style of the diagram
ROOT.gROOT.SetStyle("Plain")
ROOT.gStyle.SetOptStat(0);
ROOT.gStyle.SetOptTitle(0);
ROOT.gStyle.SetLabelSize(0.06, "xyz");
ROOT.gStyle.SetTitleSize(0.06, "xyz");
#ROOT.gStyle.SetPadRightMargin(0.1)
ROOT.gStyle.SetPadLeftMargin(0.18)
ROOT.gStyle.SetPadBottomMargin(0.18)

t = ROOT.TText()
# create ROOT histograms
hist = ROOT.TH2D("hist","Sphere:H=R",nqy,qymin,qymax, nqz, qzmin, qzmax)
hist2 = ROOT.TH2D("hist2","Sphere:H=R",nqx,qxmin,qxmax, nqy, qymin, qymax)

# and fill them with the values
for i in range(nqy):
    qy = qymin + i*stepqy
    for j in range(nqz):
        qz = qzmin + j*stepqz
        k = cvector_t(0,qy,qz)
        hist.Fill(qy,qz,(abs(ff.evaluate_for_q(k)))**2+1)

for i in range(nqy):
    qy = qymin + i*stepqy
    for j in range(nqx):
        qx = qxmin + j*stepqx
        k = cvector_t(qx,qy,0)
        hist2.Fill(qx,qy,(abs(ff.evaluate_for_q(k)))**2+1)


# create a ROOT canvas and put all plots on it
c = ROOT.TCanvas("FormfactorSphere","Formfactor Sphere", 1000,500)

c.Divide(2,1)
hist.SetMinimum(1)
hist.SetMaximum(V**2)
hist2.SetMinimum(1)
hist2.SetMaximum(V**2)
hist.SetContour(50)
hist2.SetContour(50)
#hist.GetZaxis().SetTitle(" |F|^{2} ")
#hist2.GetZaxis().SetTitle(" |F|^{2} ")


c.cd(1)
ROOT.gPad.SetLogz()
hist.GetXaxis().SetTitle(" q_{y} [nm^{-1}] ")
hist.GetXaxis().CenterTitle()
hist.GetXaxis().SetTitleOffset(1.1)
hist.GetYaxis().SetTitle(" q_{z} [nm^{-1}] ")
hist.GetYaxis().CenterTitle()
hist.GetYaxis().SetTitleOffset(1.4)
hist.GetZaxis().SetTitleOffset(1.3)
hist.Draw("col")
#t.SetNDC(1)
#t.SetTextSize(8.0e-2)
#t.DrawText(0.1, 0.01, "a")

c.cd(2)
ROOT.gPad.SetLogz()
hist2.SetMinimum(1)
hist2.GetXaxis().SetTitle(" q_{x} [nm^{-1}] ")
hist2.GetXaxis().CenterTitle()
hist2.GetXaxis().SetTitleOffset(1.1)
hist2.GetYaxis().SetTitle(" q_{y} [nm^{-1}] ")
hist2.GetYaxis().CenterTitle()
hist2.GetYaxis().SetTitleOffset(1.4)
hist2.GetZaxis().SetTitleOffset(1.3)
hist2.Draw("col")
#t.DrawText(0.1, 0.004, "b")

#c.cd(3)
#ROOT.gPad.SetLogy()
#py = hist.ProjectionX("py", fbin, lbin, 'o')
#py.GetYaxis().SetTitle(" |F|^{2} ")
#py.GetYaxis().SetTitleOffset(1.3)
#py.Draw()

#t.DrawText(0.1, 0.01, "c")

#c.cd(4)
#ROOT.gPad.SetLogy()
#px = hist2.ProjectionX("px", fbin, lbin, 'o')
#px.GetYaxis().SetTitle(" |F|^{2} ")
#px.GetYaxis().SetTitleOffset(1.3)
#px.Draw()
#t.DrawText(0.1, 0.004, "d")

c.Update()
ROOT.gApplication.Run()






#!/usr/bin/env python3

# setup_dodecahedron.py:
# Compute vertex coordinates of a regular dodecahedron.
# Auxiliary program, used during implementation of dodecahedron form factor in BornAgain.
# Joachim Wuttke, Forschungszentrum Jülich, 2016

from math import sqrt,sin,cos,pi

def distance(s,t):
    return sqrt((s[0]-t[0])**2+(s[1]-t[1])**2+(s[2]-t[2])**2)

q=sqrt(5)
# v=pi/5
# vv=2*v
c=(1+q)/4 # cos(v)
s=sqrt((5-q)/8) # sin(v)
cc=(q-1)/4 # cos(vv)
ss=sqrt((5+q)/8) # sin(vv)

def plane_rotate(k):
    t=p[k]
    ci = cc
    si = ss
    x = ci*t[0]-si*t[1]
    y = si*t[0]+ci*t[1]
    p[k+1] = [x, y,t[2]]
    p[k+4] = [x,-y,t[2]]
    ci = -c
    si = s
    x = ci*t[0]-si*t[1]
    y = si*t[0]+ci*t[1]
    p[k+2] = [x, y,t[2]]
    p[k+3] = [x,-y,t[2]]


# a=pow((7*q-15)/5, 1/3.) # edge for V=1
a=1.

rb=a/(2*s)
rd=a*c/s
R2a=(9+3*q)/8
b=a*sqrt(R2a-1/(4*s**2))
d=a*sqrt(R2a-(c/s)**2)

h=rb*c
vol = 12 * 5 * h*a*b/6
print("a=%g, R/a=%g, rb/a=%g, rd/a=%g, b/a=%g, d/a=%g" % (a, sqrt(R2a), rb/a, rd/a, b/a, d/a))
print("h=%g volume=%20.16g" % (h, vol) )

p=[None for i in range(20)]
p[ 0] = [+rb,0.,-b]
p[ 5] = [+rd,0.,-d]
p[10] = [-rd,0.,+d]
p[15] = [-rb,0.,+b]
for m in range(4):
    plane_rotate(m*5)

for i in range(len(p)):
    print("        {%20.16g*a,%20.16g*a,%20.16g*a}," % (p[i][0], p[i][1], p[i][2]))

for i in range(20):
    print("p%2i(%20.16g,%20.16g,%20.16g) at R=%20.16g" %
          (i, p[i][0], p[i][1], p[i][2], distance(p[i],[0,0,0])))

distcoll = {}
for i in range(20):
    for j in range(i+1,20):
        dist = distance(p[i],p[j])
        print("p%2i(%9.4g,%9.4g,%9.4g) - p%2i(%9.4g,%9.4g,%9.4g) = %20.17g" %
              (i, p[i][0], p[i][1], p[i][2], j, p[j][0], p[j][1], p[j][2], dist))
        dist=round(dist*1e14)/10**14
        if not dist in distcoll:
            distcoll[dist] = 1
        else:
            distcoll[dist] += 1

for dist in sorted(distcoll.keys()):
    print( "%2i times %20.16g" % (distcoll[dist],dist) )
